package com.qmth.distributed.print.api.mark;


import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.distributed.print.business.bean.result.EditResult;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.MarkQuestion;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.FieldUniqueEnum;
import com.qmth.teachcloud.common.enums.mark.MarkPaperAiMark;
import com.qmth.teachcloud.common.util.RedisUtil;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.mark.entity.*;
import com.qmth.teachcloud.mark.enums.MarkTaskStatus;
import com.qmth.teachcloud.mark.service.*;
import io.swagger.annotations.*;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * AI评卷参数表 前端控制器
 * </p>
 *
 * @author wangliang
 * @since 2025-04-10
 */
@Api(tags = "AI评卷参数Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_MARK + "/ai_question_param")
public class MarkAiQuestionParamController {
    private final static Logger log = LoggerFactory.getLogger(MarkAiQuestionParamController.class);

    @Resource
    MarkAiQuestionParamService markAiQuestionParamService;

    @Resource
    MarkAiQuestionPointService markAiQuestionPointService;

    @Resource
    MarkAiQuestionLevelService markAiQuestionLevelService;

    @Resource
    private MarkQuestionService markQuestionService;
    @Resource
    private MarkPaperService markPaperService;
    @Resource
    private MarkTaskService markTaskService;
    @Resource
    private MarkService markService;

    @Resource
    RedisUtil redisUtil;

    @ApiOperation(value = "AI评卷参数新增/修改")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "成功/失败", response = EditResult.class)})
    @Transactional
    public Result save(@ApiParam(value = "AI评卷参数信息", required = true) @Valid @RequestBody MarkAiQuestionParam markAiQuestionParam, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultUtil.error(bindingResult.getAllErrors().get(0).getDefaultMessage());
        }
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        String lockKey = SystemConstant.REDIS_AI_QUESTION_PARAM_SAVE_PREFIX + markAiQuestionParam.getQuestionId();
        boolean lock = redisUtil.lock(lockKey, SystemConstant.REDIS_LOCK_DEFAULT_TIME_OUT);
        if (!lock) {
            throw ExceptionResultEnum.ERROR.exception("正在设置AI评卷参数，请稍候再试！");
        }

        boolean update = false;
        try {
            // 有人评任务
            MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(markAiQuestionParam.getExamId(), markAiQuestionParam.getPaperNumber());
            Objects.requireNonNull(markPaper, "试卷信息为空");
            Objects.requireNonNull(markPaper.getAiMark(), "AI评卷信息为空");
            MarkQuestion markQuestion = markQuestionService.getById(markAiQuestionParam.getQuestionId());
            Objects.requireNonNull(markQuestion, "题目为空");
            if (!MarkPaperAiMark.NONE.equals(markPaper.getAiMark())
                    && (CollectionUtils.isNotEmpty(markAiQuestionParam.getPointList()) || CollectionUtils.isEmpty(markAiQuestionParam.getLevelList()))
                    && markTaskService.countByExamIdAndPaperNumberAndQuestionIdAndTaskNumberAndAiMarkedAndStatusNotIn(markAiQuestionParam.getExamId(), markAiQuestionParam.getPaperNumber(), markAiQuestionParam.getQuestionId(), 1, false, Arrays.asList(MarkTaskStatus.WAITING)) > 0) {
                throw ExceptionResultEnum.ERROR.exception("评卷员已开始评卷，请先重置评卷任务后再设置");
            }
            switch (markAiQuestionParam.getMode()) {
                case POINT://按得分点
                    if (CollectionUtils.isEmpty(markAiQuestionParam.getPointList())) {
                        throw ExceptionResultEnum.ERROR.exception("得分点信息为空");
                    }
                    // 最小分校验
                    if (markAiQuestionParam.getMinScore() == 0) {
                        throw ExceptionResultEnum.ERROR.exception("最小分必须大于0");
                    }
                    MarkAiQuestionPoint markAiQuestionPoint = markAiQuestionParam.getPointList().stream().min(Comparator.comparingDouble(MarkAiQuestionPoint::getScore)).get();
                    if (markAiQuestionParam.getMinScore() > markAiQuestionPoint.getScore()) {
                        throw ExceptionResultEnum.ERROR.exception("最小分不能超过得分点的最小分值");
                    }
                    // 各知识点得分总和校验
                    Double sumScore = markAiQuestionParam.getPointList().stream().filter(m -> m.getScore() != null).collect(Collectors.summingDouble(MarkAiQuestionPoint::getScore));
                    if (sumScore.doubleValue() - markQuestion.getTotalScore().doubleValue() != 0) {
                        throw ExceptionResultEnum.ERROR.exception("各知识点得分总和必须等于小题满分[" + markQuestion.getTotalScore() + "]分");
                    }
                    if (Objects.isNull(markAiQuestionParam.getId())) {//新增
                        markAiQuestionParam.insertInfo(sysUser.getId());
                        markAiQuestionParam.getPointList().stream().peek(s -> {
                            s.setAiQuestionId(markAiQuestionParam.getId());
                            s.setId(SystemConstant.getDbUuid());
                        }).collect(Collectors.toList());
                        markAiQuestionPointService.saveBatch(markAiQuestionParam.getPointList());
                    } else {//修改
                        markAiQuestionParam.getPointList().stream().peek(s -> {
                            if (Objects.isNull(s.getAiQuestionId())) {
                                s.setAiQuestionId(markAiQuestionParam.getId());
                            }
                            if (Objects.isNull(s.getId())) {
                                s.setId(SystemConstant.getDbUuid());
                            }
                        }).collect(Collectors.toList());

                        List<MarkAiQuestionPoint> markAiQuestionPointDbList = markAiQuestionPointService.listByAiQuestionId(markAiQuestionParam.getId());
                        List<MarkAiQuestionLevel> markAiQuestionLevelDbList = markAiQuestionLevelService.listByAiQuestionId(markAiQuestionParam.getId());

                        //删除档次的数据
                        if (CollectionUtils.isNotEmpty(markAiQuestionLevelDbList)) {
                            update = true;
                            markAiQuestionLevelService.removeByIds(markAiQuestionLevelDbList.stream().map(MarkAiQuestionLevel::getId).collect(Collectors.toList()));
                        }
                        //全删point数据全增
                        if (CollectionUtils.isNotEmpty(markAiQuestionPointDbList)) {
                            if (!CollectionUtils.isEqualCollection(markAiQuestionPointDbList, markAiQuestionParam.getPointList())) {
                                update = true;
                            }
                            markAiQuestionPointService.removeByIds(markAiQuestionPointDbList.stream().map(MarkAiQuestionPoint::getId).collect(Collectors.toList()));
                        }
                        markAiQuestionPointService.saveBatch(markAiQuestionParam.getPointList());
                        markAiQuestionParam.updateInfo(sysUser.getId());
                    }
                    break;
                case LEVEL://按档次
                    if (CollectionUtils.isEmpty(markAiQuestionParam.getLevelList())) {
                        throw ExceptionResultEnum.ERROR.exception("档次信息为空");
                    }
                    // 最小分校验
                    if (markAiQuestionParam.getMinScore() == 0) {
                        throw ExceptionResultEnum.ERROR.exception("最小分必须大于0");
                    }
                    List<MarkAiQuestionLevel> levelList = markAiQuestionParam.getLevelList();
                    levelList.sort(Comparator.comparing(MarkAiQuestionLevel::getMinScore));
                    Double minRangeScore = 0D;
                    for (MarkAiQuestionLevel aiQuestionLevel : levelList) {
                        if(minRangeScore.doubleValue() - aiQuestionLevel.getMinScore().doubleValue() != 0){
                            throw ExceptionResultEnum.ERROR.exception("区间分数必须从0开始，且区间分数段连续");
                        } else {
                            minRangeScore = aiQuestionLevel.getMaxScore();
                        }
                    }
                    // 档位最大分校验
                    Double maxScore = markAiQuestionParam.getLevelList().stream().filter(m -> m.getMaxScore() != null).max(Comparator.comparingDouble(MarkAiQuestionLevel::getMaxScore)).get().getMaxScore();
                    if (maxScore.doubleValue() - markQuestion.getTotalScore().doubleValue() != 0) {
                        throw ExceptionResultEnum.ERROR.exception("档位最大分必须等于小题满分[" + markQuestion.getTotalScore() + "]分");
                    }
                    if (Objects.isNull(markAiQuestionParam.getId())) {//新增
                        markAiQuestionParam.insertInfo(sysUser.getId());
                        markAiQuestionParam.getLevelList().stream().peek(s -> {
                            s.setAiQuestionId(markAiQuestionParam.getId());
                            s.setId(SystemConstant.getDbUuid());
                        }).collect(Collectors.toList());
                        markAiQuestionLevelService.saveBatch(markAiQuestionParam.getLevelList());
                    } else {
                        markAiQuestionParam.getLevelList().stream().peek(s -> {
                            if (Objects.isNull(s.getAiQuestionId())) {
                                s.setAiQuestionId(markAiQuestionParam.getId());
                            }
                            if (Objects.isNull(s.getId())) {
                                s.setId(SystemConstant.getDbUuid());
                            }
                        }).collect(Collectors.toList());

                        List<MarkAiQuestionPoint> markAiQuestionPointDbList = markAiQuestionPointService.listByAiQuestionId(markAiQuestionParam.getId());
                        List<MarkAiQuestionLevel> markAiQuestionLevelDbList = markAiQuestionLevelService.listByAiQuestionId(markAiQuestionParam.getId());

                        //删除得分点的数据
                        if (CollectionUtils.isNotEmpty(markAiQuestionPointDbList)) {
                            update = true;
                            markAiQuestionPointService.removeByIds(markAiQuestionPointDbList.stream().map(MarkAiQuestionPoint::getId).collect(Collectors.toList()));
                        }
                        //全删level数据全增
                        if (CollectionUtils.isNotEmpty(markAiQuestionLevelDbList)) {
                            if (!CollectionUtils.isEqualCollection(markAiQuestionLevelDbList, markAiQuestionParam.getLevelList())) {
                                update = true;
                            }
                            markAiQuestionLevelService.removeByIds(markAiQuestionLevelDbList.stream().map(MarkAiQuestionLevel::getId).collect(Collectors.toList()));
                        }
                        markAiQuestionLevelService.saveBatch(markAiQuestionParam.getLevelList());
                        markAiQuestionParam.updateInfo(sysUser.getId());
                    }
                    break;
                default:
                    break;
            }
//            if (update) {
//                markQuestionService.updateClearOcrResult(markAiQuestionParam.getQuestionId(), true);
//            }
            markAiQuestionParamService.saveOrUpdate(markAiQuestionParam);
            markQuestionService.updateAiMark(markAiQuestionParam.getExamId(), markAiQuestionParam.getPaperNumber(), markAiQuestionParam.getQuestionId());

            if (markAiQuestionParamService.existMarkAiQuestionPointOrLevel(markAiQuestionParam.getExamId(), markAiQuestionParam.getPaperNumber(), markAiQuestionParam.getQuestionId())
                    && markTaskService.countByExamIdAndPaperNumberAndQuestionIdAndTaskNumberAndAiMarkedAndStatusNotIn(markAiQuestionParam.getExamId(), markAiQuestionParam.getPaperNumber(), markAiQuestionParam.getQuestionId(), 1, false, null) > 0) {
                markService.deleteMarkTask(markQuestion, true, true);
                markService.checkStudentSubjectiveScore(markAiQuestionParam.getExamId(), markPaper.getCoursePaperId());
            }
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof DuplicateKeyException) {
                String errorColumn = e.getCause().toString();
                String columnStr = errorColumn.substring(errorColumn.lastIndexOf("key") + 3, errorColumn.length()).replaceAll("'", "");
                throw ExceptionResultEnum.SQL_ERROR.exception("[" + FieldUniqueEnum.convertToTitle(columnStr) + "]数据不允许重复插入");
            } else if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, ((ApiException) e).getCode(), e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        } finally {
            redisUtil.releaseLock(lockKey);
        }
        return ResultUtil.ok(true);
    }

    @ApiOperation(value = "AI评卷参数查询")
    @RequestMapping(value = "/info", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "AI评卷参数信息", response = MarkAiQuestionParam.class)})
    public Result info(@ApiParam(value = "题目id", required = true) @RequestParam Long questionId) {
        MarkAiQuestionParam markAiQuestionParam = markAiQuestionParamService.getByExamIdAndPaperNumberAndQuestionId(null, null, questionId);
        if (Objects.nonNull(markAiQuestionParam)) {
            List<MarkAiQuestionPoint> markAiQuestionPointDbList = markAiQuestionPointService.listByAiQuestionId(markAiQuestionParam.getId());
            List<MarkAiQuestionLevel> markAiQuestionLevelDbList = markAiQuestionLevelService.listByAiQuestionId(markAiQuestionParam.getId());
            markAiQuestionParam.setListInfo(markAiQuestionPointDbList, markAiQuestionLevelDbList);
        }
        return ResultUtil.ok(markAiQuestionParam);
    }
}
